@@ -19,8 +19,9 @@ from ..aggregation.aggregation_interface import AbstractSeriesAggregator
 try:
     # The efficient c version of the LTTB algorithm
     from .algorithms.lttb_c import LTTB_core_c as LTTB_core
-except:
+except (ImportError, ModuleNotFoundError):
     import warnings
+
     warnings.warn("Could not import lttbc; will use a (slower) python alternative.")
     from .algorithms.lttb_py import LTTB_core_py as LTTB_core
 
@@ -75,7 +76,7 @@ class LTTB(AbstractSeriesAggregator):
         super().__init__(
             interleave_gaps,
             nan_position,
-            dtype_regex_list=[rf"{dtype}\d*" for dtype in ["float", "int", "uint"]]
+            dtype_regex_list=[rf"{dtype}\d*" for dtype in ("float", "int", "uint")]
             + ["category", "bool"],
         )
 
@@ -277,7 +278,7 @@ class EfficientLTTB(AbstractSeriesAggregator):
         super().__init__(
             interleave_gaps,
             nan_position,
-            dtype_regex_list=[rf"{dtype}\d*" for dtype in ["float", "int", "uint"]]
+            dtype_regex_list=[rf"{dtype}\d*" for dtype in ("float", "int", "uint")]
             + ["category", "bool"],
         )
 
@@ -286,7 +287,7 @@ class EfficientLTTB(AbstractSeriesAggregator):
         ratio_threshold = 100
 
         # TODO -> test this with a move of the .so file
-        if LTTB_core.__name__ == 'LTTB_core_py':
+        if LTTB_core.__name__ == "LTTB_core_py":
             size_threshold = 1_000_000
 
         if s.shape[0] > size_threshold and s.shape[0] / n_out > ratio_threshold:
